/* slg (SLice & Glue) -- spliting big files and gluing slices
 * Copyright (C) 1994-1998 Robert Fenk
 * Version: $Version$
 *
 * Ein guter Freund (Juergen) und ich waren genervt unter Unix "split"
 * und unter DOS "copy" verwenden zu muessen, falls wir mal Dateien
 * hatten die groesser waren als das was auf eine Diskette drauf geht.
 * Daraufhin waren wir motiviert genug in ein paar Stunden die
 * Grundversion dieses kleine Tools zu erstellen. Es sollte auf jeder 
 * Platform laufen und mehr Usability als die anderen Tools bieten.
 *
 * Nach und nach kam 
 * Dieses Jahr kam noch die CRC Pruefsummenberechnung hinzu und
 * damit sollte es erstmal fertig sein!
 *
 * The CRC code is derived from the public domain gzip written
 * by Jean-loup Gailly.
 *
 * See the license_msg below and the file COPYING for the software license.
 */

#define VERSION "$Version$"

static char *rcsid = "$Id: slg.c,v 1.1 2002/01/15 20:43:09 fenk Exp $";

static char *license_msg =
"   SLG version $Version$, Copyright (C) 1992-1998 Robert Fenk\n"
"   This program is free software; you can redistribute it and/or modify\n"
"   it under the terms of the GNU General Public License as published by\n"
"   the Free Software Foundation; either version 2, or (at your option)\n"
"   any later version.\n"
"\n"
"   This program is distributed in the hope that it will be useful,\n"
"   but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
"   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
"   GNU General Public License for more details.\n"
"\n"
"   You should have received a copy of the GNU General Public License\n"
"   along with this program; if not, write to the Free Software\n"
"   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n"
 ;

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>


/* The makro 'unix' is normally predefined on all UNIX systems? */
#ifdef unix
#  include <unistd.h>
#  define DIRSEP "/"
#  define BUFSIZE 180000L
#else
#  define DIRSEP "\\"
#  ifdef __WIN32__
#    define BUFSIZE 180000L
#  else
#    define BUFSIZE 60000L
#  endif
#endif

typedef unsigned long uLong;

int MAXNR = 100;		/* maximum number of slices */
long DISKSIZE = 1440000UL;	/* default disksize (1406KB) */
int RESTRICT = 0;		/* use filenames with DOS-conventions */
int KEEPSRC = 0;		/* keep sources after slicing/gluing */
char *SLICESUFFIX = "s%02lu";	/* suffix for the slices */
char *SDFSUFFIX = "sdf";	/* suffix for the slice-description-file */
char ARGV0[256];

enum Error {
  ERR_cant_open_file = 2,
  ERR_cant_write_file = 4,
  ERR_cant_remove_file = 8,
  ERR_slice_size_differs = 16,
  ERR_slice_crc_differs = 32,
  ERR_not_neccessary = 64,
  ERR_no_idea = 128,
  ERR_unknown_option = 129,
  ERR_out_of_memory = 130,
  ERR_too_many = 131,
} ERRORS;

void
setError(int code)
{
  ERRORS |= code;
  fprintf(stderr, "%s: ", ARGV0);
}

/***********************************************************************
 * upcase --
 * Upcases the string s
 */
char *
upcase(char *s)
{
  int i;

  for (i = 0; s[i]; i++) {
    if ('a' <= s[i] && s[i] <= 'z') {
      s[i] -= 32;
    }
  }
  s[i] = '\0';

  return s;
}

/***********************************************************************
 * fileDir --
 * Returns a name comprised of all of the path components in path
 * excluding the last element.
 *
 * Arguments:
 * path -                the path
 * dir -                 directory of path, has to be big enough to hold
 *                       the directory
 * 
 * Returnvalue:
 * dir
 *
 * Sideeffects:  
 * A copy of the directory of path is stored in dir.
 */
char *
fileDir(char *path, char *dir)
{
  char *ds = strrchr(path, *DIRSEP);
  int len = 0;
  
  if (!ds && path[1] == ':') {
    ds = path + 2;		/* DOS drive */
  }
  if (ds) {
    len = (int) (ds - path) + 1;
  }
  strncpy(dir, path, len);
  dir[len] = '\0';
  return dir;
}

/***********************************************************************
 * fileBase -- 
 * Returns the base of a file
 *
 * Arguments:
 * path -                the path
 * base -                base of path, has to be big enough to hold
 *                       the base
 * 
 * Returnvalue:
 * base
 *
 * Sideeffects:  
 * A copy of the base of path is stored in base.
 */
char *
fileBase(char *path, char *base)
{
  char *ds = strrchr(path, *DIRSEP);
  char *dot;

  if (ds) {
    strcpy(base, ds + 1);
  }
  else {
    strcpy(base, path);
  }

  dot = strrchr(base, '.');
  if (dot) {
    *dot = '\0';
  }

  return base;
}

/***********************************************************************
 * fileTail --
 * Returns all of the characters in path after the last directory
 * separator. If path contains no separators then returns path.
 *
 * Arguments:
 * path -                the path
 *
 * Returnvalue:
 * Pointer to the start of tail in path
 */
char *
fileTail(char *path)
{
  char *ds = strrchr(path, *DIRSEP);
  if (ds) {
    return ds + 1;
  }
  else {
    return path;
  }
}

/***********************************************************************
 * fileExt -- 
 * Returns the extension of a file
 *
 * Arguments:
 * path -                the path
 * ext -                 extension of path, has to be big enough to hold
 *                       the extension
 *
 * Returnvalue:
 * ext
 *
 * Sideeffects:  
 * A copy of the extension of path is stored in extension.
 */
char *
fileExt(char *path, char *ext)
{
  char *dot = strrchr(path, '.');
  if (dot) {
    strcpy(ext, dot + 1);
  }
  else {
    ext[0] = '\0';
  }
  return ext;
}

/***********************************************************************
 * fileClean -- 
 * Replaces all none numerical & none alphabetical charcters in s with "_"
 *
 * Arguments:
 * path -                the path
 */
char *
fileClean(char *s)
{
  char c, *sp;

  for (sp = s, c = *sp; *sp; sp++, c = *sp) {
    if (!(('0' <= c && c <= '9') ||
	  ('A' <= c && c <= 'Z') ||
	  ('a' <= c && c <= 'z'))) {
      *sp = '_';
    }
  }


  return s;
}

/***********************************************************************
 * fileJoin --
 * Concatenates directory, basename and extension to a path
 */
char *
fileJoin(char *path, char *dir, char *base, char *ext)
{
  path[0] = '\0';
  strcat(path, dir);
  if (strlen(path) && (path[strlen(path) - 1] != *DIRSEP)) {
    strcat(path, DIRSEP);
  }

  if (RESTRICT) {
    fileClean(base);
    base[9] = '\0';
    strncat(path, base, 9);
  }
  else {
    strcat(path, base);
  }

  if (strlen(ext)) {
    strcat(path, ".");
    strcat(path, ext);
  }

  if (RESTRICT) {
    upcase(path + strlen(dir));
  }

  return path;
}

/***********************************************************************
 * fileDOS -- 
 * Restrict filename to DOS conventions
 */
char *
fileDOS(char *path, char *newpath)
{
  char dir[256], base[256], ext[256];
  fileDir(path, dir);
  fileBase(path, base);
  fileExt(path, ext);
  base[8] = '\0';
  ext[3] = '\0';
  fileClean(upcase(base));
  fileClean(upcase(ext));
  fileJoin(newpath, dir, base, ext);
  return newpath;
}

/***********************************************************************
 * openFile --
 * Tries to open a file
 */
FILE *
openFile(char *file, char *mode)
{
  FILE *fd;
  char answer[256], upFile[256];

  strcpy(upFile, file);
  upcase(upFile);

  fd = fopen(file, mode);
  if (!fd) {
    fd = fopen(upFile, mode);
  }

  while (!fd) {
    setError(ERR_cant_open_file);
    fprintf(stderr, "can not open file '%s'\n"
	    "Try again (change disk, etc.)? (y/n) ", file);

    switch (*gets(answer)) {
    case 'n':
    case 'N':
      return NULL;
    }

    fd = fopen(file, mode);
    if (!fd) {
      fd = fopen(upFile, mode);
    }
  }

  return fd;
}


/***********************************************************************
 * Table of CRC-32's of all single-byte values (made by makecrc.c)
 */
uLong crc_32_tab[] =
{
  0x00000000L, 0x77073096L, 0xee0e612cL, 0x990951baL, 0x076dc419L,
  0x706af48fL, 0xe963a535L, 0x9e6495a3L, 0x0edb8832L, 0x79dcb8a4L,
  0xe0d5e91eL, 0x97d2d988L, 0x09b64c2bL, 0x7eb17cbdL, 0xe7b82d07L,
  0x90bf1d91L, 0x1db71064L, 0x6ab020f2L, 0xf3b97148L, 0x84be41deL,
  0x1adad47dL, 0x6ddde4ebL, 0xf4d4b551L, 0x83d385c7L, 0x136c9856L,
  0x646ba8c0L, 0xfd62f97aL, 0x8a65c9ecL, 0x14015c4fL, 0x63066cd9L,
  0xfa0f3d63L, 0x8d080df5L, 0x3b6e20c8L, 0x4c69105eL, 0xd56041e4L,
  0xa2677172L, 0x3c03e4d1L, 0x4b04d447L, 0xd20d85fdL, 0xa50ab56bL,
  0x35b5a8faL, 0x42b2986cL, 0xdbbbc9d6L, 0xacbcf940L, 0x32d86ce3L,
  0x45df5c75L, 0xdcd60dcfL, 0xabd13d59L, 0x26d930acL, 0x51de003aL,
  0xc8d75180L, 0xbfd06116L, 0x21b4f4b5L, 0x56b3c423L, 0xcfba9599L,
  0xb8bda50fL, 0x2802b89eL, 0x5f058808L, 0xc60cd9b2L, 0xb10be924L,
  0x2f6f7c87L, 0x58684c11L, 0xc1611dabL, 0xb6662d3dL, 0x76dc4190L,
  0x01db7106L, 0x98d220bcL, 0xefd5102aL, 0x71b18589L, 0x06b6b51fL,
  0x9fbfe4a5L, 0xe8b8d433L, 0x7807c9a2L, 0x0f00f934L, 0x9609a88eL,
  0xe10e9818L, 0x7f6a0dbbL, 0x086d3d2dL, 0x91646c97L, 0xe6635c01L,
  0x6b6b51f4L, 0x1c6c6162L, 0x856530d8L, 0xf262004eL, 0x6c0695edL,
  0x1b01a57bL, 0x8208f4c1L, 0xf50fc457L, 0x65b0d9c6L, 0x12b7e950L,
  0x8bbeb8eaL, 0xfcb9887cL, 0x62dd1ddfL, 0x15da2d49L, 0x8cd37cf3L,
  0xfbd44c65L, 0x4db26158L, 0x3ab551ceL, 0xa3bc0074L, 0xd4bb30e2L,
  0x4adfa541L, 0x3dd895d7L, 0xa4d1c46dL, 0xd3d6f4fbL, 0x4369e96aL,
  0x346ed9fcL, 0xad678846L, 0xda60b8d0L, 0x44042d73L, 0x33031de5L,
  0xaa0a4c5fL, 0xdd0d7cc9L, 0x5005713cL, 0x270241aaL, 0xbe0b1010L,
  0xc90c2086L, 0x5768b525L, 0x206f85b3L, 0xb966d409L, 0xce61e49fL,
  0x5edef90eL, 0x29d9c998L, 0xb0d09822L, 0xc7d7a8b4L, 0x59b33d17L,
  0x2eb40d81L, 0xb7bd5c3bL, 0xc0ba6cadL, 0xedb88320L, 0x9abfb3b6L,
  0x03b6e20cL, 0x74b1d29aL, 0xead54739L, 0x9dd277afL, 0x04db2615L,
  0x73dc1683L, 0xe3630b12L, 0x94643b84L, 0x0d6d6a3eL, 0x7a6a5aa8L,
  0xe40ecf0bL, 0x9309ff9dL, 0x0a00ae27L, 0x7d079eb1L, 0xf00f9344L,
  0x8708a3d2L, 0x1e01f268L, 0x6906c2feL, 0xf762575dL, 0x806567cbL,
  0x196c3671L, 0x6e6b06e7L, 0xfed41b76L, 0x89d32be0L, 0x10da7a5aL,
  0x67dd4accL, 0xf9b9df6fL, 0x8ebeeff9L, 0x17b7be43L, 0x60b08ed5L,
  0xd6d6a3e8L, 0xa1d1937eL, 0x38d8c2c4L, 0x4fdff252L, 0xd1bb67f1L,
  0xa6bc5767L, 0x3fb506ddL, 0x48b2364bL, 0xd80d2bdaL, 0xaf0a1b4cL,
  0x36034af6L, 0x41047a60L, 0xdf60efc3L, 0xa867df55L, 0x316e8eefL,
  0x4669be79L, 0xcb61b38cL, 0xbc66831aL, 0x256fd2a0L, 0x5268e236L,
  0xcc0c7795L, 0xbb0b4703L, 0x220216b9L, 0x5505262fL, 0xc5ba3bbeL,
  0xb2bd0b28L, 0x2bb45a92L, 0x5cb36a04L, 0xc2d7ffa7L, 0xb5d0cf31L,
  0x2cd99e8bL, 0x5bdeae1dL, 0x9b64c2b0L, 0xec63f226L, 0x756aa39cL,
  0x026d930aL, 0x9c0906a9L, 0xeb0e363fL, 0x72076785L, 0x05005713L,
  0x95bf4a82L, 0xe2b87a14L, 0x7bb12baeL, 0x0cb61b38L, 0x92d28e9bL,
  0xe5d5be0dL, 0x7cdcefb7L, 0x0bdbdf21L, 0x86d3d2d4L, 0xf1d4e242L,
  0x68ddb3f8L, 0x1fda836eL, 0x81be16cdL, 0xf6b9265bL, 0x6fb077e1L,
  0x18b74777L, 0x88085ae6L, 0xff0f6a70L, 0x66063bcaL, 0x11010b5cL,
  0x8f659effL, 0xf862ae69L, 0x616bffd3L, 0x166ccf45L, 0xa00ae278L,
  0xd70dd2eeL, 0x4e048354L, 0x3903b3c2L, 0xa7672661L, 0xd06016f7L,
  0x4969474dL, 0x3e6e77dbL, 0xaed16a4aL, 0xd9d65adcL, 0x40df0b66L,
  0x37d83bf0L, 0xa9bcae53L, 0xdebb9ec5L, 0x47b2cf7fL, 0x30b5ffe9L,
  0xbdbdf21cL, 0xcabac28aL, 0x53b39330L, 0x24b4a3a6L, 0xbad03605L,
  0xcdd70693L, 0x54de5729L, 0x23d967bfL, 0xb3667a2eL, 0xc4614ab8L,
  0x5d681b02L, 0x2a6f2b94L, 0xb40bbe37L, 0xc30c8ea1L, 0x5a05df1bL,
  0x2d02ef8dL
};


/***********************************************************************
 * updateCRC -- 
 * function & table are from gzip.
 * the used 32-bit polynom is:
 * x^32+x^26+x^23+x^22+x^16+x^12+x^11+x^10+x^8+x^7+x^5+x^4+x^2+x+1
 *
 * Arguments:
 * buf -         buffer
 * len -         length of buffer
 * crc -         crc of the previous buffers
 * 
 * Returnvalue:  
 * crc of buffer
 */
uLong
updateCRC(unsigned char *buf, uLong len, uLong crc)
{
  register uLong c;

  c = crc;
  if (len) {
    do {
      c = crc_32_tab[((int) c ^ (*buf++)) & 0xff] ^ (c >> 8);
    }
    while (--len);
  }

  return c ^ 0xffffffffL;
}


/***********************************************************************
 * slice --
 * Slices a big file.
 * 
 * Arguments:
 * file -                filename
 * destPath -            destination directory
 */
int
slice(char *file, char *destPath)
{
  unsigned char buffer[BUFSIZE];	/* buffer for io */

  char SDfile[256], baseName[256], extName[256], sliceName[256], dosName[256];

  uLong crc;
  long nr, count, size;

  FILE *fdSrc, *fdDest, *fdSDF;

  /* generate the filenames */

  fileDOS(file, dosName);
  fileBase(file, baseName);
  fileExt(file, extName);

  /* open source file and check size */
  if ((fdSrc = openFile(file, "rb")) == NULL) {
    return ERR_cant_open_file;
  }

  fseek(fdSrc, 0, SEEK_END);
  if ((size = ftell(fdSrc)) < DISKSIZE) {
    setError(ERR_not_neccessary);
    fprintf(stderr,
	    "it is not neccessary to slice a file"
	    " that is smaller than %ldKB. '%s' has %ld Bytes.",
	    DISKSIZE, baseName, size);

    fclose(fdSrc);
    return ERR_not_neccessary;
  }

  fseek(fdSrc, 0, SEEK_SET);

  /* calculate number of slices and open slicedescription file */

  nr = 1 + size / DISKSIZE;
  printf("\nYou will need %ld disks with %ld kB.\n",
	 nr, DISKSIZE / 1024);
  if (nr > MAXNR) {
    setError(ERR_too_many);
    fprintf(stderr, "sorry too many files\n");
    return ERR_too_many;
  }

  printf("Slicing file '%s' (One HashMark = %ldkB).",
	 file, BUFSIZE / 1024);

  fileJoin(SDfile, destPath, baseName, SDFSUFFIX);

  if ((fdSDF = fopen(SDfile, "w")) == NULL) {
    setError(ERR_cant_write_file);
    fprintf(stderr,
	    "can not write slice description file '%s'\n", SDfile);
    return ERR_cant_write_file;
  }

  if (*destPath) {
    printf("\nWriting files to directory '%s'\n", destPath);
  }

  printf("\n Slicedescription file '%s'.\n", fileTail(SDfile));

  fprintf(fdSDF, "%ld\n", nr);
  fprintf(fdSDF, "%s\n%s\n", fileTail(file), fileTail(dosName));
  fprintf(fdSDF, "\nSlices\t\t\tSize\tCRC\n");

  /* slicing the file */

  for (nr = 0, count = 1; nr < MAXNR && count; nr++) {
    /* open slice */
    sprintf(extName, SLICESUFFIX, nr);
    fileJoin(sliceName, destPath, baseName, extName);
    printf("\n Writing slice '%s': ", fileTail(sliceName));
    if ((fdDest = openFile(sliceName, "wb")) == NULL) {
      return ERR_cant_open_file;
    }

    /* one slice is DISKSIZE bytes long; initialise crc */
    for (crc = 0xffffffffL, size = 0; size < DISKSIZE && count;) {
      /* read from source */
      if (size + sizeof(buffer) <= DISKSIZE) {
	count = fread(buffer, 1, sizeof(buffer), fdSrc);
      }
      else {
	count = fread(buffer, 1, DISKSIZE - size, fdSrc);
      }

      size += count;

      if (count) {
	/* calculate crc and write to slice */
	crc = updateCRC(buffer, count, crc);
	fwrite(buffer, 1, (int) count, fdDest);
	printf("#");
	fflush(stdout);
      }
    }

    /* store information about slice in slice-description-file */
    fprintf(fdSDF, "%-23s %7ld %lu\n", fileTail(sliceName), size, crc);

    for (size = DISKSIZE / BUFSIZE - size / BUFSIZE; size > 1L; size--)
      printf(" ");

    printf("\t CRC=%lu", crc);

    fclose(fdDest);
  }

  fclose(fdSrc);
  fclose(fdSDF);

  printf("\n\n %ld Silces made.\n", nr);

  if (!KEEPSRC) {
    if (unlink(file) != 0) {
      setError(ERR_cant_remove_file);
      fprintf(stderr, "cannot remove '%s'", file);
      return ERR_cant_remove_file;
    }
  }

  return 0;
}


/***********************************************************************
 * glue --
 * Glues slices to a big file.
 * 
 * Arguments:
 * SDfile -              slice-description-file
 * destPath -            destination directory
 */
int
glue(char *SDfile, char *destPath)
{
  unsigned char buffer[BUFSIZE];	/* buffer for io */
  char dirName[256], baseName[256], extName[256];
  char sliceName[256], destName[256], dosName[256], destFile[256];

  long max, nr, count, crc, orgcrc, size, orgsize;

  FILE *fdSrc, *fdDest, *fdSDF;

  /* generate the filenames */
  fileDir(SDfile, dirName);
  fileBase(SDfile, baseName);
  extName[0] = '\0';

  /* open slice-description-file */
  if ((fdSDF = openFile(SDfile, "r")) == NULL) {
    char answer[256];
    printf("\nCan not read slice-description-file '%s'!", SDfile);
    printf("\nTry to glue with out slice-description-file. (y/n) ");

    switch (*gets(answer)) {
    case 'n':
    case 'N':
      return ERR_no_idea;
    }

    max = 0;
    strcpy(extName, "xxx");
    strcpy(destName, baseName);
    strcpy(dosName, baseName);
  }
  else {
    /* read number of slices, destination filename and
       destination filename with DOS conventions */
    fscanf(fdSDF, "%ld\n%s\n%s\n", &max, destName, dosName);
    fgets((char *) buffer, 256, fdSDF);
  }

  if (RESTRICT) {
    fileJoin(destFile, destPath, dosName, "");
  }
  else {
    fileJoin(destFile, destPath, destName, "");
  }

  /* open destination */
  if ((fdDest = fopen(destFile, "wb")) == NULL) {
    setError(ERR_cant_write_file);
    fprintf(stderr, "can't write file '%s'\n", destPath);
    return ERR_cant_write_file;
  }

  if (max) {
    printf("\n%ld slices are needed!\n", max);
  }
  else {
    printf("\n%ld slices will be glue if necessary!\n", MAXNR);
    max = MAXNR;
  }

  printf("Writing file '%s' (One HashMark = %ldkB).\n",
	 destFile, BUFSIZE / 1024);

  /* gluing the file */

  for (nr = 0; nr < max; nr++) {
    if (fdSDF) {
      /* read slicename, original size and crc from the
	 slice-description-file */
      fscanf(fdSDF, "%s\t%ld\n%lu\n", baseName, &orgsize, &orgcrc);
    }
    else {
      sprintf(extName, SLICESUFFIX, nr);
    }

    /* open slice */
    fileJoin(sliceName, dirName, baseName, extName);
    if ((fdSrc = openFile(sliceName, "rb")) == NULL) {
      return ERR_cant_open_file;
    }

    /* read slice and write to destination */
    printf("\n Gluing file '%s' : ", fileTail(sliceName));

    for (size = 0, crc = 0xffffffffL, count = 1; count;) {
      count = fread(buffer, 1, sizeof(buffer), fdSrc);
      if (count) {
	size += count;
	/* calculate crc and write to destination file */
	crc = updateCRC(buffer, count, crc);
	fwrite(buffer, 1, (int) count, fdDest);
	printf("#");
	fflush(stdout);
      }
    }

    /* check correctness of slice */
    if (!fdSDF) {
      for (size = DISKSIZE / BUFSIZE - size / BUFSIZE; size > 1; size--)
	printf(" ");
      printf("\t CRC=%lu Size=%ld", crc, size);
    }
    else if (orgsize != size) {
      setError(ERR_slice_size_differs);
      fprintf(stderr,
	      "size differs form original by %ld bytes",
	      (long) orgsize - (long) size);
    }
    else if (crc != orgcrc) {
      setError(ERR_slice_crc_differs);
      fprintf(stderr,
	      "CRC differs (%lu!=%lu)!", orgcrc, crc);
    }
    else {
      for (size = DISKSIZE / BUFSIZE - size / BUFSIZE; size > 1; size--)
	printf(" ");
      printf("\t CRC=%lu", crc);
    }

    fclose(fdSrc);
  }

  fclose(fdDest);

  printf("\n\n %ld Slices glued.\n", nr);

  if (fdSDF) {
    fseek(fdSDF, 0L, SEEK_SET);
    fscanf(fdSDF, "%ld\n%s\n%s\n", &max, destName, dosName);
    fgets((char *) buffer, 256, fdSDF);
    extName[0] = '\0';
  }

  if (!KEEPSRC) {
    while (nr) {
      nr--;
      if (fdSDF) {
	fscanf(fdSDF, "%s\t%ld\n%lu\n", baseName, &orgsize, &orgcrc);
      }
      else {
	sprintf(extName, "s%02lu", nr);
      }
      fileJoin(sliceName, dirName, baseName, extName);
      if (unlink(sliceName) != 0) {
	setError(ERR_cant_remove_file);
	fprintf(stderr,
		"can't remove file '%s', aborting", sliceName);
	return ERR_cant_remove_file;
      }
    }

    fclose(fdSDF);

    if (fdSDF) {
      if (unlink(SDfile) != 0) {
	setError(ERR_cant_remove_file);
	fprintf(stderr,
		"can't remove file '%s', aborting!", SDfile);
	return ERR_cant_remove_file;
      }
    }
  }
  else {
    fclose(fdSDF);
  }

  return 0;
}


/***********************************************************************
 * main -- 
 * parses commandline arguments and call 
 */
int
main(int argc, char *argv[])
{
  enum Job {
    NONE, SLICE, GLUE
  } job = NONE;
  int i;
  char *argv0;
  char *destPath = "";
  struct stat statbuf;

  argv0 = strstr(argv[0], DIRSEP);

  if (!argv0) {
    argv0 = argv[0];
  }

  strcpy(ARGV0, argv0 + 1);

  /* check programm name for slice/glue */
  upcase(argv[0]);
  if (strstr(argv[0], "SLICE")) {
    job = SLICE;
  }
  else if (strstr(argv[0], "GLUE")) {
    job = GLUE;
  }
  else {
    job = NONE;
  }

  if (argc > 2) {
    if (stat(argv[argc - 1], &statbuf) == 0) {
#ifdef unix
      i = S_ISDIR(statbuf.st_mode);
#else
      i = S_IFDIR & statbuf.st_mode;
#endif
      if (i) {
	argc--;
	destPath = argv[argc];
      }
    }
  }


  /* parse arguments */
  for (i = 1; i < argc; i++) {
    if (argv[i][0] != '-') {
      switch (job) {
      case SLICE:
	slice(argv[i], destPath);
	break;
      case GLUE:
	glue(argv[i], destPath);
	break;
      default:
	setError(ERR_no_idea);
	fprintf(stderr,
		"no idea how to handle arg: %s\n", argv[i]);
      }
      continue;
    }

    switch (argv[i][1]) {
    case 'g':
      job = GLUE;
      break;

    case 's':
      job = SLICE;
      break;

    case 'd':
      DISKSIZE = atoi(argv[++i]) * 1024;
      break;

    case 'k':
      KEEPSRC = 1;
      break;

    case 'r':
      RESTRICT = 1;
      break;

    case 'l':
      printf("\n%s", license_msg);
      break;

    case 'h':
      printf("\nSLice & Glue Version %s by Robert Fenk!\n", VERSION);
      printf("\n Running on a %s machine using '%s' as separator"
	     "\n and a buffersize of %ldKB!\n",
	     *DIRSEP == '/' ? "unix" : "DOS", DIRSEP, BUFSIZE / 1024);

      printf("\n Usage: slg <options> -s <files...> [<dest-dir>]"
	     "\n        slg <options> -g <slice-description-files...> [<dest-dir>]"
	     "\n\n SLice & Glue provides you with the capability to"
	     "\n split a big file into disk (1.44MB) right parts and"
	     "\n glue the slices some where else in a comfortable and "
	     "\n safe (CRC) way."
	     "\n\n Common options:"
	     "\n -k          keep the input files"
	     "\n -r          restrict output filenames to DOS conventions"
	     "\n -d <kb>     sets disksize in KB (default 1406)"
	     "\n -l          print license information"
	     "\n -h          print help"
	     "\n\n"
	);
      break;

    case 'M':
      MAXNR = atoi(argv[++i]);
      break;

    case 'S':
      SLICESUFFIX = argv[++i];
      break;

    case 'D':
      SDFSUFFIX = argv[++i];
      break;

    default:
      setError(ERR_unknown_option);
      fprintf(stderr, "unknown option '%s'", argv[i]);
    }
  }

  printf("\n");
  return ERRORS;
}
